// Copyright © 2010 Loïc Molinari <loicm@loicm.com>
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEFACE_H_
#define DEFACE_H_

#include "deface_glext.h"
#include "deface_programs.h"
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gudev/gudev.h>

enum {
  DEFACE_TEXTURE_FRAME_PATTERN = 0,
  DEFACE_NR_TEXTURES
};

// GLX extension functions.
typedef struct {
  // SGIX_fbconfig
  ChooseFBConfigFunc choose_fb_config;
  GetVisualFromFBConfigFunc get_visual_from_fb_config;
  // SGI_swap_control
  SwapIntervalFunc swap_interval;
  // SGI_video_sync
  GetVideoSyncFunc get_video_sync;
  WaitVideoSyncFunc wait_video_sync;
} GlxExtensions;

// OpenGL extension functions.
typedef struct {
  // ARB_fragment_program
  GenProgramsFunc gen_programs;
  DeleteProgramsFunc delete_programs;
  ProgramStringFunc program_string;
  BindProgramFunc bind_program;
  ProgramLocalParameter4fvFunc program_local_parameter_4fv;
  GetProgramivFunc get_program_iv;
  // ARB_vertex_buffer_object & ARB_pixel_buffer_object
  BindBufferFunc bind_buffer;
  DeleteBuffersFunc delete_buffers;
  GenBuffersFunc gen_buffers;
  BufferDataFunc buffer_data;
  BufferSubDataFunc buffer_sub_data;
  MapBufferFunc map_buffer;
  UnmapBufferFunc unmap_buffer;
} OpenglExtensions;

typedef struct {
  union {
    struct {
      gfloat x;
      gfloat y;
      gfloat width;
      gfloat height;
    };
    gfloat v[4];
  };
} DefaceRect;

typedef struct {
  guint8 r;
  guint8 g;
  guint8 b;
} DefaceColor;

typedef struct {
  DefaceColor focused;
  DefaceColor unfocused;
} DefaceTheme;

typedef struct {
  gfloat positions[68*2];
  guint8 colors[68*4];
  gfloat texcoords[68*2];
  guint16 indices[42*3];
} DefaceVertexBuffer;

typedef struct {
  DefaceVertexBuffer vb;
  DefaceRect geometry;
  gint32 quit;
  GThread* thread;
  GdkDisplay* display;
  GdkScreen* screen;
  GdkWindow* window;
  gint32 xscreen_nr;
  XVisualInfo* xvisual;
  Display* xdisplay;
  Window xwindow;
  GlxExtensions glx;
  GLXContext xcontext;
  gint32 glx_major;
  gint32 glx_minor;
  DefaceTheme theme;
  OpenglExtensions gl;
  guint32 texture_id[DEFACE_NR_TEXTURES];
  guint32 program_id[DEFACE_NR_PROGRAMS];
  guint16 flags;
} DefaceUI;

typedef struct {
  guint16 width;
  guint16 height;
  guint16 framerate_n;
  guint16 framerate_d;
} DefaceCameraFormat;

typedef struct {
  GUdevDeviceNumber devnum;  // Device number
  gchar* product;            // Device product description
  gchar* devpath;            // Device udev path
  gchar* devfile;            // Device file
  GArray* formats;           // Supported formats
  guint16 size;              // Deface camera data size
} DefaceCameraPlugged;

typedef struct {
  gint32 quit;
  GThread* thread;
  GUdevClient* udev;
  GPtrArray* cameras;
} DefaceCamera;

typedef struct {
#ifndef NDEBUG
  GThread* main_thread;
#endif  // NDEBUG
  gboolean argb_visual;
  const gchar* error_message;
  DefaceUI ui;
  DefaceCamera camera;
} Deface;

gboolean deface_ui_spawn_thread (Deface* deface);
void deface_ui_join_thread (Deface* deface);
gboolean deface_camera_spawn_thread (Deface* deface);
void deface_camera_join_thread (Deface* deface);
void deface_quit (Deface* deface, const gchar* error);

#endif  // DEFACE_H_
