// Copyright © 2010 Loïc Molinari <loicm@loicm.com>
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif  // HAVE_CONFIG_H

#include "deface.h"
#include "deface_log.h"
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <asm/types.h>
#include <linux/videodev2.h>

// static GArray* get_supported_formats (DefaceContext* context,
//                                       const gchar* devfile) {
//   gint32 fd = open (devfile, O_RDWR | O_NONBLOCK, 0);
//   if (fd > 1) {
//     gint32 i = 0;
//     struct v4l2_fmtdesc format;
//     while (TRUE) {
//       format.index = i;
//       format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
//       if (ioctl (fd, VIDIOC_ENUM_FMT, &format) > -1) {
//         g_print ("%d: '%s'\n", format.index, format.description);
//       } else if (errno == EINVAL) {
//         break;
//       } else {
//         return NULL;
//       }
//       i++;
//     }
//     return (GArray*) 1;
//   } else {
//     g_printerr ("Cannot open '%s': %d, %s\n",
//                 devfile, errno, strerror (errno));
//     return NULL;
//   }
// }

// static DefaceCamera* camera_new (DefaceContext* context, GUdevDevice* dev,
//                                  const gchar* devfile) {
//   GArray* formats = get_supported_formats (context, devfile);
//   if (formats != NULL) {
//     const GUdevDeviceNumber devnum = g_udev_device_get_device_number (dev);
//     const gchar* product = g_udev_device_get_property (dev, "ID_V4L_PRODUCT");
//     const gchar* devpath = g_udev_device_get_property (dev, "DEVPATH");
//     const guint16 product_len = strlen (product) + 1;
//     const guint16 devpath_len = strlen (devpath) + 1;
//     const guint16 devfile_len = strlen (devfile) + 1;
//     const guint16 size = product_len + devpath_len + devfile_len +
//                          sizeof (DefaceCamera);

//     DefaceCamera* camera = g_slice_alloc (size);
//     camera->devnum = devnum;
//     camera->product = (gchar*) camera + sizeof (DefaceCamera);
//     camera->devpath = camera->product + product_len;
//     camera->devfile = camera->devpath + devpath_len;
//     camera->formats = formats;
//     camera->size = size;

//     memcpy (camera->product, product, product_len);
//     memcpy (camera->devpath, devpath, devpath_len);
//     memcpy (camera->devfile, devfile, devfile_len);

//     return camera;
//   } else {
//     return NULL;
//   }
// }

// static void camera_free (DefaceCamera* camera) {
//   g_slice_free1 (camera->size, camera);
// }

// static void add_camera (DefaceContext* context, GUdevDevice* dev) {
//   const gchar* devfile = g_udev_device_get_device_file (dev);
//   if (devfile != NULL) {
//     // Discard non-video devices.
//     if (strstr (devfile, "video") != NULL) {
//       // Discard devices that don't support V4L or V4L2.
//       gint32 version =
//           g_udev_device_get_property_as_int (dev, "ID_V4L_VERSION");
//       if (version == 2 || version == 0) {
//         // Discard devices that don't have capture capabilities.
//         if (strstr (g_udev_device_get_property (dev, "ID_V4L_CAPABILITIES"),
//                     ":capture:") != NULL) {
//           // Store the camera device if it's supported.
//           DefaceCamera* camera = camera_new (context, dev, devfile);
//           if (camera != NULL)
//             g_ptr_array_add (context->cameras, camera);
//         }
//       }
//     }
//   }
// }

// static void remove_camera (DefaceContext* context, GUdevDevice* dev) {
//   const GUdevDeviceNumber devnum = g_udev_device_get_device_number (dev);
//   GPtrArray* cameras = context->cameras;
//   const guint32 nr_cameras = cameras->len;
//   for (guint32 i = 0; i < nr_cameras; i++) {
//     DefaceCamera* camera = (DefaceCamera*) cameras->pdata[i];
//     if ((guint64) camera->devnum == (guint64) devnum) {
//       g_ptr_array_remove_index_fast (context->cameras, i);
//       camera_free (camera);
//       break;
//     }
//   }
// }

// static void udev_uevent_cb (GUdevClient* client, const gchar* action,
//                             GUdevDevice* device, DefaceContext* context) {
//   if (!strcmp (action, "add")) {
//     add_camera (context, device);
//   } else if (!strcmp (action, "remove")) {
//     remove_camera (context, device);
//   }
// }

static gpointer camera_main (Deface* deface) {
  DEFACE_DLOG (DEFACE_INFO, "entered camera thread");
  DefaceCamera* camera = &deface->camera;

  while (!g_atomic_int_get (&camera->quit)) {
    usleep (100000);
  }

  DEFACE_DLOG (DEFACE_INFO, "leaving camera thread");
  return NULL;
}

gboolean deface_camera_spawn_thread (Deface* deface) {
  // const gchar* const subsystems[] = { "video4linux", NULL };
  // GList* devices;
  // GList* walk;

  // context->cameras = g_ptr_array_new ();
  // context->udev = g_udev_client_new (subsystems);
  // g_signal_connect (G_OBJECT (context->udev), "uevent",
  //                   G_CALLBACK (udev_uevent_cb), context);

  // devices = g_udev_client_query_by_subsystem (context->udev, "video4linux");
  // walk = devices;
  // while (walk != NULL) {
  //   add_camera (context, G_UDEV_DEVICE (walk->data));
  //   g_object_unref (walk->data);
  //   walk = walk->next;
  // }

  GThread* thread;
  GError* error;
  thread = g_thread_create ((GThreadFunc) camera_main, deface, TRUE, &error);
  if (thread != NULL) {
    deface->camera.thread = thread;
    return TRUE;
  } else {
    DEFACE_LOG (DEFACE_ERROR, "can't spawn camera thread: %s", error->message);
    return FALSE;
  }
}

void deface_camera_join_thread (Deface* deface) {
  // GPtrArray* cameras = context->cameras;
  // const guint32 nr_cameras = cameras->len;
  // for (guint32 i = 0; i < nr_cameras; i++)
  //   camera_free ((DefaceCamera*) cameras->pdata[i]);
  // g_ptr_array_free (context->cameras, TRUE);
  // g_object_unref (context->udev);

  if (deface->camera.thread != NULL) {
    g_atomic_int_set (&deface->camera.quit, 1);
    DEFACE_DLOG (DEFACE_INFO, "joining camera thread");
    g_thread_join (deface->camera.thread);
    deface->camera.thread = NULL;
    DEFACE_DLOG (DEFACE_INFO, "joined camera thread");
  }
}
