// Copyright © 2010 Loïc Molinari <loicm@loicm.com>
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEFACE_GLEXT_H_
#define DEFACE_GLEXT_H_

#include <GL/gl.h>
#define GLX_GLXEXT_PROTOTYPES
#include <GL/glx.h>
#include <GL/glxext.h>
#undef GLX_GLXEXT_PROTOTYPES
#include <stddef.h>

// GLX SGIX_fbconfig extension.
typedef GLXFBConfig* (*ChooseFBConfigFunc) (Display*, int, const int*, int*);
typedef XVisualInfo* (*GetVisualFromFBConfigFunc) (Display*, GLXFBConfig);

// GLX SGI_swap_control extension.
typedef int (*SwapIntervalFunc) (int);

// GLX SGI_video_sync extension.
typedef int (*GetVideoSyncFunc) (unsigned int*);
typedef int (*WaitVideoSyncFunc) (int, int, unsigned int*);

// OpenGL ARB_vertex_program extension.
typedef void (*GenProgramsFunc) (GLsizei, GLuint*);
typedef void (*DeleteProgramsFunc) (GLsizei, const GLuint*);
typedef void (*ProgramStringFunc) (GLenum, GLenum, GLsizei, const GLvoid*);
typedef void (*BindProgramFunc) (GLenum, GLuint);
typedef void (*ProgramLocalParameter4fvFunc) ( GLenum, GLuint, const GLfloat*);
typedef void (*GetProgramivFunc) (GLenum, GLenum, GLint*);
#define GL_PROGRAM_LENGTH_ 0x8627
#define GL_PROGRAM_STRING_ 0x8628
#define GL_PROGRAM_ERROR_POSITION_ 0x864b
#define GL_PROGRAM_BINDING_ 0x8677
#define GL_PROGRAM_ERROR_STRING_ 0x8874
#define GL_PROGRAM_FORMAT_ASCII_ 0x8875
#define GL_PROGRAM_FORMAT_ 0x8876
#define GL_PROGRAM_INSTRUCTIONS_ 0x88a0
#define GL_MAX_PROGRAM_INSTRUCTIONS_ 0x88a1
#define GL_PROGRAM_NATIVE_INSTRUCTIONS_ 0x88a2
#define GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ 0x88a3
#define GL_PROGRAM_TEMPORARIES_ 0x88a4
#define GL_MAX_PROGRAM_TEMPORARIES_ 0x88a5
#define GL_PROGRAM_NATIVE_TEMPORARIES_ 0x88a6
#define GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ 0x88a7
#define GL_PROGRAM_PARAMETERS_ 0x88a8
#define GL_MAX_PROGRAM_PARAMETERS_ 0x88a9
#define GL_PROGRAM_NATIVE_PARAMETERS_ 0x88aa
#define GL_MAX_PROGRAM_NATIVE_PARAMETERS_ 0x88ab
#define GL_PROGRAM_ATTRIBS_ 0x88ac
#define GL_MAX_PROGRAM_ATTRIBS_ 0x88ad
#define GL_PROGRAM_NATIVE_ATTRIBS_ 0x88ae
#define GL_MAX_PROGRAM_NATIVE_ATTRIBS_ 0x88af
#define GL_PROGRAM_ADDRESS_REGISTERS_ 0x88b0
#define GL_MAX_PROGRAM_ADDRESS_REGISTERS_ 0x88b1
#define GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ 0x88b2
#define GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ 0x88b3
#define GL_MAX_PROGRAM_LOCAL_PARAMETERS_ 0x88b4
#define GL_MAX_PROGRAM_ENV_PARAMETERS_ 0x88b5
#define GL_PROGRAM_UNDER_NATIVE_LIMITS_ 0x88b6

// OpenGL ARB_fragment_program extension (shares symbols with
// ARB_vertex_program).
#define GL_FRAGMENT_PROGRAM_ 0x8804
#define GL_PROGRAM_ALU_INSTRUCTIONS_ 0x8805
#define GL_PROGRAM_TEX_INSTRUCTIONS_ 0x8806
#define GL_PROGRAM_TEX_INDIRECTIONS_ 0x8807
#define GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ 0x8808
#define GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ 0x8809
#define GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ 0x880a
#define GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ 0x880b
#define GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ 0x880c
#define GL_MAX_PROGRAM_TEX_INDIRECTIONS_ 0x880d
#define GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ 0x880e
#define GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ 0x880f
#define GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ 0x8810
#define GL_MAX_TEXTURE_COORDS_ 0x8871
#define GL_MAX_TEXTURE_IMAGE_UNITS_ 0x8872

// OpenGL ARB_vertex_buffer_object extension.
typedef void (*BindBufferFunc) (GLenum, GLuint);
typedef void (*DeleteBuffersFunc) (GLsizei, const GLuint *);
typedef void (*GenBuffersFunc) (GLsizei, GLuint *);
typedef void (*BufferDataFunc) (GLenum, ptrdiff_t, const GLvoid *, GLenum);
typedef void (*BufferSubDataFunc) (GLenum, ptrdiff_t, ptrdiff_t,
                                   const GLvoid *);
typedef GLvoid* (*MapBufferFunc) (GLenum, GLenum);
typedef GLboolean (*UnmapBufferFunc) (GLenum);
#define GL_BUFFER_SIZE_ 0x8764
#define GL_BUFFER_USAGE_ 0x8765
#define GL_ARRAY_BUFFER_ 0x8892
#define GL_ELEMENT_ARRAY_BUFFER_ 0x8893
#define GL_ARRAY_BUFFER_BINDING_ 0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING_ 0x8895
#define GL_VERTEX_ARRAY_BUFFER_BINDING_ 0x8896
#define GL_NORMAL_ARRAY_BUFFER_BINDING_ 0x8897
#define GL_COLOR_ARRAY_BUFFER_BINDING_ 0x8898
#define GL_INDEX_ARRAY_BUFFER_BINDING_ 0x8899
#define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ 0x889a
#define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ 0x889b
#define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ 0x889c
#define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ 0x889d
#define GL_WEIGHT_ARRAY_BUFFER_BINDING_ 0x889e
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ 0x889f
#define GL_READ_ONLY_ 0x88b8
#define GL_WRITE_ONLY_ 0x88b9
#define GL_READ_WRITE_ 0x88ba
#define GL_BUFFER_ACCESS_ 0x88bb
#define GL_BUFFER_MAPPED_ 0x88bc
#define GL_BUFFER_MAP_POINTER_ 0x88bd
#define GL_STREAM_DRAW_ 0x88e0
#define GL_STREAM_READ_ 0x88e1
#define GL_STREAM_COPY_ 0x88e2
#define GL_STATIC_DRAW_ 0x88e4
#define GL_STATIC_READ_ 0x88e5
#define GL_STATIC_COPY_ 0x88e6
#define GL_DYNAMIC_DRAW_ 0x88e8
#define GL_DYNAMIC_READ_ 0x88e9
#define GL_DYNAMIC_COPY_ 0x88ea

// OpenGL ARB_pixel_buffer_object extension (shares symbols with
// ARB_vertex_buffer_object).
#define GL_PIXEL_PACK_BUFFER_ 0x88eb
#define GL_PIXEL_UNPACK_BUFFER_ 0x88ec
#define GL_PIXEL_PACK_BUFFER_BINDING_ 0x88ed
#define GL_PIXEL_UNPACK_BUFFER_BINDING_ 0x88ef

#endif  // DEFACE_GLEXT_H_
